#-----------------------------------------------------------
# 
# Table Manager
# Copyright (C) 2008-2009 Borys Jurgiel
# 
#-----------------------------------------------------------
# 
# licensed under the terms of GNU GPL 2
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, print to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
#---------------------------------------------------------------------

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *
from tableManagerUi import Ui_Dialog
from tableManagerUiRename import Ui_Rename
from tableManagerUiClone import Ui_Clone
from tableManagerUiInsert import Ui_Insert



########## CLASS DialogRename ##############################

class DialogRename(QDialog, Ui_Rename):
  def __init__(self, iface, fields, selection):
    QDialog.__init__(self)
    self.iface = iface
    self.setupUi(self)
    self.fields = fields
    self.selection = selection
    self.setWindowTitle('Rename field: '+fields[selection].name())
    self.lineEdit.setValidator(QRegExpValidator(QRegExp('\w{,10}'),self))
    self.lineEdit.setText(fields[selection].name())

  def accept(self):
    if self.newName() == self.fields[self.selection].name():
      QDialog.reject(self)
      return
    for i in self.fields.values():
      if self.newName() == i.name():
        QMessageBox.warning(self,'Rename field','There is another field with the same name.\nPlease type different one.')
        return
    if not self.newName():
      QMessageBox.warning(self,'Rename field','The new name cannot be empty')
      self.lineEdit.setText(self.fields[self.selection].name())
      return
    QDialog.accept(self)

  def newName(self):
    return self.lineEdit.text()



########## CLASS DialogClone ##############################

class DialogClone(QDialog, Ui_Clone):
  def __init__(self, iface, fields, selection):
    QDialog.__init__(self)
    self.iface = iface
    self.setupUi(self)
    self.fields = fields
    self.selection = selection
    self.setWindowTitle('Clone field: '+fields[selection].name())
    self.comboDsn.addItem('at the first position')
    for i in range(len(fields)):
      self.comboDsn.addItem('after the '+ fields[i].name() + ' field')
    self.comboDsn.setCurrentIndex(selection+1)
    self.lineDsn.setValidator(QRegExpValidator(QRegExp('\w{,10}'),self))
    self.lineDsn.setText(fields[selection].name()[:8] + '_2')

  def accept(self):
    if not self.result()[1]:
      QMessageBox.warning(self,'Clone field','The new name cannot be empty')
      return
    if self.result()[1] == self.fields[self.selection].name():
        QMessageBox.warning(self,'Clone field','The new field\'s name must be different then source\'s one!')
        return
    for i in self.fields.values():
      if self.result()[1] == i.name():
        QMessageBox.warning(self,'Clone field','There is another field with the same name.\nPlease type different one.')
        return
    QDialog.accept(self)

  def result(self):
    return self.comboDsn.currentIndex(), self.lineDsn.text()



########## CLASS DialogInsert ##############################

class DialogInsert(QDialog, Ui_Insert):
  def __init__(self, iface, fields, selection):
    QDialog.__init__(self)
    self.iface = iface
    self.setupUi(self)
    self.fields = fields
    self.selection = selection
    self.setWindowTitle('Insert field')
    self.lineName.setValidator(QRegExpValidator(QRegExp('\w{,10}'),self))
    self.comboType.addItem('Integer')
    self.comboType.addItem('Real')
    self.comboType.addItem('String')
    self.comboPos.addItem('at the first position')
    for i in range(len(fields)):
      self.comboPos.addItem('after the '+ fields[i].name() + ' field')
    self.comboPos.setCurrentIndex(selection+1)

  def accept(self):
    if not self.result()[0]:
      QMessageBox.warning(self,'Insert new field','The new name cannot be empty')
      return
    for i in self.fields.values():
      if self.result()[0] == i.name():
        QMessageBox.warning(self,'Insert new field','There is another field with the same name.\nPlease type different one.')
        return
    QDialog.accept(self)

  def result(self):
    return self.lineName.text(), self.comboType.currentIndex(), self.comboPos.currentIndex()



########## CLASS TableManager ##############################

class TableManager(QDialog, Ui_Dialog):

  def __init__(self, iface):
    QDialog.__init__(self)
    self.iface = iface
    self.setupUi(self)
    self.layer = self.iface.activeLayer()
    self.provider = self.layer.dataProvider()
    self.fields = self.provider.fields()
    self.isUnsaved = False  # No unsaved changes yet
    if self.provider.storageType() == 'ESRI Shapefile': # Is provider saveable?
      self.isSaveable = True
    else:
      self.isSaveable = False
    self.needsRedraw = True # Preview table is redrawed only on demand. This is for initial drawing.
    self.selection = -1     # Don't highlight any field on startup

    QObject.connect(self.butUp, SIGNAL('clicked()'), self.doMoveUp)
    QObject.connect(self.butDown, SIGNAL('clicked()'), self.doMoveDown)
    QObject.connect(self.butDel, SIGNAL('clicked()'), self.doDelete)
    QObject.connect(self.butIns, SIGNAL('clicked()'), self.doInsert)
    QObject.connect(self.butClone, SIGNAL('clicked()'), self.doClone)
    QObject.connect(self.butRename, SIGNAL('clicked()'), self.doRename)
    QObject.connect(self.butConvert, SIGNAL('clicked()'), self.doConvert)
    QObject.connect(self.butCalc, SIGNAL('clicked()'), self.doCalc)
    QObject.connect(self.butSave, SIGNAL('clicked()'), self.doSave)
    QObject.connect(self.butSaveAs, SIGNAL('clicked()'), self.doSaveAs)
    QObject.connect(self.buttonBox, SIGNAL('rejected()'), self.doDone)
    QObject.connect(self.fieldsTable, SIGNAL('itemSelectionChanged ()'), self.selectionChanged)
    QObject.connect(self.tabWidget, SIGNAL('currentChanged (int)'), self.drawDataTable)

    self.setWindowTitle('Table Manager: '+self.layer.name())
    self.progressBar.reset()
    self.restoreCfg()
    self.drawFieldsTable()
    self.readData()



  def drawFieldsTable(self): # Draws the fields table on startup and redraws it when changed
    fields = self.fields
    self.fieldsTable.setRowCount(0)
    for i in range(len(fields)):
      self.fieldsTable.setRowCount(i+1)
      item = QTableWidgetItem(fields[i].name())
      item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
      self.fieldsTable.setItem(i,0,item)
      item = QTableWidgetItem(fields[i].typeName())
      item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
      self.fieldsTable.setItem(i,1,item)
      self.fieldsTable.setColumnWidth(0, 128)
      self.fieldsTable.setColumnWidth(1, 64)



  def readData(self): # Reads data from the 'provider' QgsDataProvider into the 'data' list [[column1] [column2] [column3]...]
    fields = self.fields
    self.data = []
    for i in range(len(fields)):
      self.data += [[]]
    steps = self.provider.featureCount()
    stepp = steps / 10
    if stepp == 0:
      stepp = 1
    progress = 'Reading data ' # As a progress bar is used the main window's status bar, because the own one is not initialized yet
    n = 0
    self.provider.select(self.provider.attributeIndexes())
    feat = QgsFeature()
    while self.provider.nextFeature(feat):
      attr = feat.attributeMap()
      for i in range(len(attr)):
        self.data[i] += [attr[i].toString()]
      n += 1
      if n % stepp == 0:
        progress += '|' 
        self.iface.mainWindow().statusBar().showMessage(QString(progress))
    self.iface.mainWindow().statusBar().showMessage('')



  def drawDataTable(self,tab): # Called when user switches tabWidget to the Table Preview
    if tab != 1 or self.needsRedraw == False: return
    fields = self.fields
    self.dataTable.clear()
    self.repaint()
    self.dataTable.setColumnCount(len(fields))
    self.dataTable.setRowCount(self.provider.featureCount())
    header = QStringList()
    for i in fields.values():
      header.append(i.name())
    self.dataTable.setHorizontalHeaderLabels(header)
    self.progressBar.setRange (0, len(self.data)+1)
    self.progressBar.setFormat(QString('Drawing table: %p%'))
    formatting = True
    if formatting: # slower procedure, with formatting the table items
      for i in range(len(self.data)):
        self.progressBar.setValue(i+1)
        for j in range(len(self.data[i])):
          item = QTableWidgetItem(unicode(self.data[i][j]))
          item.setFlags(Qt.ItemIsSelectable)
          if fields[i].type() == 6 or fields[i].type() == 2:
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
          self.dataTable.setItem(j,i,item)
    else: # about 25% faster procedure, without formatting
      for i in range(len(self.data)):
        self.progressBar.setValue(i+1)
        for j in range(len(self.data[i])):
          self.dataTable.setItem(j,i,QTableWidgetItem(unicode(self.data[i][j])))
    self.dataTable.resizeColumnsToContents()
    self.needsRedraw = False
    self.progressBar.reset()



  def setChanged(self): # Called after making any changes
    if self.isSaveable:
      self.butSave.setEnabled(True)
    self.butSaveAs.setEnabled(True)
    self.isUnsaved = True       # data are unsaved
    self.needsRedraw = True     # preview table needs to redraw



  def selectionChanged(self): # Called when user is changing field selection of field
    self.selection = self.fieldsTable.currentRow()
    item = self.selection
    if item == -1:
      self.butUp.setEnabled(False)
      self.butDown.setEnabled(False)
      self.butRename.setEnabled(False)
      self.butDel.setEnabled(False)
      self.butClone.setEnabled(False)
      self.butConvert.setEnabled(False)
      self.butCalc.setEnabled(False)
    else:
      if item == 0:
        self.butUp.setEnabled(False)
      else:
        self.butUp.setEnabled(True)
      if item == self.fieldsTable.rowCount()-1:
        self.butDown.setEnabled(False)
      else:
        self.butDown.setEnabled(True)
      if self.fieldsTable.rowCount() == 1:
        self.butDel.setEnabled(False)
      else:
        self.butDel.setEnabled(True)
      if self.fields[item].type() in [2,6,10]:
         self.butRename.setEnabled(True)
         self.butClone.setEnabled(True)
 #       self.butConvert.setEnabled(True)
 #       self.butCalc.setEnabled(True)
      else:
        self.butRename.setEnabled(False)
        self.butClone.setEnabled(False)
        self.butConvert.setEnabled(False)
        self.butCalc.setEnabled(False)



  def doMoveUp(self): # Called when appropriate button was pressed
    item = self.selection
    tmp = self.fields[item]
    self.fields[item] = self.fields[item-1]
    self.fields[item-1] = tmp
    for i in range(0,2):
      tmp = QTableWidgetItem(self.fieldsTable.item(item,i))
      self.fieldsTable.setItem(item,i,QTableWidgetItem(self.fieldsTable.item(item-1,i)))
      self.fieldsTable.setItem(item-1,i,tmp)
    if item > 0:
      self.fieldsTable.setCurrentCell(item-1,0)
    tmp = self.data[item]
    self.data[item]=self.data[item-1]
    self.data[item-1]=tmp
    self.setChanged()



  def doMoveDown(self): # Called when appropriate button was pressed
    item = self.selection
    tmp = self.fields[item]
    self.fields[self.selection] = self.fields[self.selection+1]
    self.fields[self.selection+1] = tmp
    for i in range(0,2):
      tmp = QTableWidgetItem(self.fieldsTable.item(item,i))
      self.fieldsTable.setItem(item,i,QTableWidgetItem(self.fieldsTable.item(item+1,i)))
      self.fieldsTable.setItem(item+1,i,tmp)
    if item < self.fieldsTable.rowCount()-1:
      self.fieldsTable.setCurrentCell(item+1,0)
    tmp = self.data[item]
    self.data[item]=self.data[item+1]
    self.data[item+1]=tmp
    self.setChanged()



  def doRename(self): # Called when appropriate button was pressed
    dlg = DialogRename(self.iface,self.fields,self.selection)
    if dlg.exec_() == QDialog.Accepted:
      newName = dlg.newName()
      self.fields[self.selection].setName(newName)
      item = self.fieldsTable.item(self.selection,0)
      item.setText(newName)
      self.fieldsTable.setItem(self.selection,0,item)
      self.fieldsTable.setColumnWidth(0, 128)
      self.fieldsTable.setColumnWidth(1, 64)
      self.setChanged()



  def doDelete(self): # Called when appropriate button was pressed
    warning = 'Are you sure you want to remove the field %s ?' % self.fields[self.selection].name()
    if QMessageBox.warning(self, 'Delete field', warning , QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
      return
    del(self.data[self.selection])
    del(self.fields[self.selection])
    self.fields = dict(zip(range(len(self.fields)), self.fields.values()))
    self.drawFieldsTable()
    self.setChanged()



  def doInsert(self): # Called when appropriate button was pressed
    dlg = DialogInsert(self.iface,self.fields,self.selection)
    if dlg.exec_() == QDialog.Accepted:
      (aName, aType, aPos) = dlg.result()
      if aType == 0:
        aLength = 10
        aPrec = 0
        aVariant = QVariant.Int
        aTypeName = 'Integer'
      elif aType == 1:
        aLength = 32
        aPrec = 3
        aVariant = QVariant.Double
        aTypeName = 'Real'
      else:
        aLength = 80
        aPrec = 0
        aVariant = QVariant.String
        aTypeName = 'String'
      self.data += [[]]
      if aPos < len(self.fields):
        fieldsToMove = range(aPos+1,len(self.fields)+1)
        fieldsToMove.reverse()
        for i in fieldsToMove:
          self.fields[i] = self.fields[i-1]
          self.data[i] = self.data[i-1]
      self.fields[aPos] = QgsField(aName, aVariant, aTypeName, aLength, aPrec, QString())
      aData = []
      if aType == 2:
        aItem = ""
      else:
        aItem = 0
      for i in range(len(self.data[0])):
        aData += [aItem]
      self.data[aPos] = aData
      self.drawFieldsTable()
      self.fieldsTable.setCurrentCell(aPos,0)
      self.setChanged()



  def doClone(self): # Called when appropriate button was pressed
    dlg = DialogClone(self.iface,self.fields,self.selection)
    if dlg.exec_() == QDialog.Accepted:
      (dst, newName) = dlg.result()
      self.data += [[]]
      movedField = QgsField(self.fields[self.selection])
      movedData = self.data[self.selection]
      if dst < len(self.fields):
        fieldsToMove = range(dst+1,len(self.fields)+1)
        fieldsToMove.reverse()
        for i in fieldsToMove:
          self.fields[i] = self.fields[i-1]
          self.data[i] = self.data[i-1]
      self.fields[dst] = movedField
      self.fields[dst].setName(newName)
      self.data[dst] = movedData
      self.drawFieldsTable()
      self.fieldsTable.setCurrentCell(dst,0)
      self.setChanged()



  def doConvert(self): # Called when appropriate button was pressed
    #comboBox for the destination type. Nizej wynik testu, progressBar i klawisze Konwertuj/Anuluj
    #dlg = DialogConvert(self.iface,self.fields,self.selection) 
    #if dlg.exec_() == QDialog.Accepted:
      #(result, srcType, dstType) = dlg.result()
    ### TO DO
    self.setChanged()



  def doCalc(self): # Called when appropriate button was pressed
    ### TO DO
    self.setChanged()



  def doSave(self): # Called when appropriate button was pressed
    # I believe the procedure below is as much safe as it's only possible...

    ##temporary checking
    for i in self.fields:
      for j in self.fields:
        if self.fields[i].name().toUpper() == self.fields[j].name().toUpper() and self.fields[i]!=self.fields[j]:
          QMessageBox.warning(self, 'Table Manager', 'The names of two fields (%s and %s) are the same.\nQuantum GIS isn\'t fully case-sensitive yet, so you have to type unique field names, without distinguishing lower and upper case.' % (self.fields[i].name(), self.fields[j].name()))
          return
    ##end of temporary checking

    tmpDir = QDir.tempPath()
    srcPath = self.provider.dataSourceUri()
    if srcPath.right(4).toUpper() == '.SHP': # if the path points to the shp file, remove the extension...
      srcPath.chop(4)
      srcName = srcPath.right(srcPath.size() - srcPath.lastIndexOf('/') - 1)
    else: # ...but if it points only to a directory, try to determine the name of the file inside!
      qPath = QDir(srcPath)
      qPath.setNameFilters(['*.shp', '*.SHP'])
      if len(qPath.entryList()) == 1: # there is exactly one shapefile inside
        srcName = qPath.entryList()[0]
        srcName.chop(4)
        srcPath += '/' + srcName 
      else:
        QMessageBox.warning(self, 'Table Manager', 'I cannot determine the layer source file: ' + srcPath + " !\nThe layer won't be changed, please use the Save As button.")
        return
    # write the layer to the temporary directory
    if self.writeToShp(tmpDir+'/'+srcName+'.shp', self.provider.encoding()) != 0:
      QMessageBox.warning(self, 'Table Manager', 'Failed saving the changes to the temporary directory: ' + tmpDir + " !\nThe layer won't be changed, please use the Save As button.")
      QgsVectorFileWriter.deleteShapeFile(tmpDir+'/'+srcName+'.shp')
      return
    # try to remove the old .dbf~ backup
    QFile(srcPath+'.dbf~').remove()
    # rename the oryginal .dbf file to .dbf~
    if not QFile(srcPath+'.dbf').rename(srcPath+'.dbf~'):
      QMessageBox.warning(self, 'Table Manager', 'Failed backuping the old table to '+srcPath+".dbf~\nThe layer won't be changed, please use the Save As button.")
      QgsVectorFileWriter.deleteShapeFile(tmpDir+'/'+srcName+'.shp')
      return
    # copy the .dbf from the temp directory to the target location
    if not QFile(tmpDir+'/'+srcName+'.dbf').copy(srcPath+'.dbf'):
      # something went wrong. Restore the dbf~ file
      if not QFile(srcPath+'.dbf~').rename(srcPath+'.dbf'):
        QMessageBox.warning(self, 'Table Manager', 'WARNING! I can neither save the new '+srcPath+'.dbf file\nnor restore it from the '+srcPath+'.dbf~ backup.\nPlease check it manually!')
        return
      QMessageBox.warning(self, 'Table Manager', 'Failed saving the changes to '+srcPath+".dbf\nThe layer will not be changed, please use the Save As button.")
      QgsVectorFileWriter.deleteShapeFile(tmpDir+'/'+srcName+'.shp')
      return
    # clean the temp directory
    QgsVectorFileWriter.deleteShapeFile(tmpDir+'/'+srcName+'.shp')
    # create the new layer
    layerName = self.layer.name()
    newLayer = QgsVectorLayer(srcPath+'.shp', layerName, 'ogr')
    if not newLayer.isValid():
      QMessageBox.warning(self, 'Table Manager', "WARNING! The changes seem to be commited, but I can't reload the layer!\nPlease check it out!\nThe old table is backuped as "+srcName+'.dbf~.')
      return
    # copy the symbology (it's possible only if the clasyfying field was not moved nor changed!)
    if QMessageBox.question(self, 'Saving successful','Saving successful. The old table has been backuped as '+srcName+".dbf~. Do you wish to keep the old symbology? Note that if the clasyfying field was moved, all objects can become invisible. In that case please adjust the symbology manually.", QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
      newLayer.copySymbologySettings(self.layer)
    # reload the layer
    QgsMapLayerRegistry.instance().removeMapLayer(self.layer.getLayerID())
    QgsMapLayerRegistry.instance().addMapLayer(newLayer)
    # point the self.layer to the new one.
    self.layer = self.iface.activeLayer()
    self.provider = self.layer.dataProvider()
    self.fields = self.provider.fields()
    self.butSave.setEnabled(False)
    self.saveCfg() # save cfg only in case of any successfull operations, not when user just play ;)
    self.isUnsaved = False



  def doSaveAs(self): # Called when appropriate button was pressed
    ##temporary checking
    for i in self.fields:
      for j in self.fields:
        if self.fields[i].name().toUpper() == self.fields[j].name().toUpper() and self.fields[i]!=self.fields[j]:
          QMessageBox.warning(self, 'Table Manager', 'The names of two fields (%s and %s) are the same.\nQuantum GIS isn\'t fully case-sensitive yet, so you have to type unique field names, without distinguishing lower and upper case.' % (self.fields[i].name(), self.fields[j].name()))
          return
    ##end of temporary checking

    filters = 'Shapefiles (*.shp)'
    fileDialog = QgsEncodingFileDialog(self, 'Save as', self.lastDirectory, filters, self.lastEncoding)
    fileDialog.setAcceptMode(QFileDialog.AcceptSave)
    if fileDialog.exec_() != QDialog.Accepted:
      return
    fileName = unicode(fileDialog.selectedFiles().first())
    encoding = str(fileDialog.encoding())
    if not fileName:
      return
    filePath = QFileInfo(fileName).absoluteFilePath()
    if filePath.isEmpty():
      return
    if QFileInfo(filePath).suffix().toUpper() != "SHP":
      QDir().mkpath(filePath)
      filePath = filePath + "/" + QFileInfo(filePath).fileName() + ".shp"

    # if saving to the source layer...
    reloadSourceLayer = False
    if filePath == self.provider.dataSourceUri():
      if QMessageBox.question(self, 'Table Manager', 'You are attemping to save the changes to the original file. Are you sure you want to do this? If yes, the original layer will be removed from the legend.', QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
        reloadSourceLayer = True
      else:
        return

    if self.writeToShp(filePath,encoding) != 0:
      return

    # formatting TOC item name and adding to the TOC
    layer = QgsVectorLayer(filePath, QFileInfo(filePath).completeBaseName(), 'ogr')
    if layer.isValid():
      QgsMapLayerRegistry.instance().addMapLayer(layer)
    else:
      QMessageBox.warning(self, 'Table Manager', "WARNING! The new layer seems to be created, but is invalid.\nIt won't be loaded.")
    if reloadSourceLayer:
      QgsMapLayerRegistry.instance().removeMapLayer(self.layer.getLayerID())
      self.layer = self.iface.activeLayer()
      self.provider = self.layer.dataProvider()
      self.fields = self.provider.fields()
      self.setWindowTitle('Table Manager: '+self.layer.name())
    self.isUnsaved = False
    self.lastDirectory = QFileInfo(fileName).absolutePath()
    self.lastEncoding = encoding
    self.saveCfg() # Store configuration only in case of any successfull operations, not when user is just playing ;)



  def writeToShp(self, filePath, encoding): # write data to a shapefile
    if QFile(filePath).exists():
      try:
        QgsVectorFileWriter.deleteShapeFile(filePath)
      except:
        QMessageBox.warning(self, 'Table Manager', 'Cannot overwrite an existing shapefile.\nPlease remove it manually.')
        return 1
    self.progressBar.setRange(0, self.provider.featureCount())
    self.progressBar.setFormat(QString('Saving data: %p%'))
    self.progressBar.setValue(0)
    # create destination layer
    self.provider.select(self.provider.attributeIndexes())
    writer = QgsVectorFileWriter(filePath, encoding, self.fields, self.provider.geometryType(), self.provider.crs())
    if writer.hasError():
      QMessageBox.warning(self, 'Table Manager', 'Error creating file. Check permissions.')
      self.progressBar.reset()
      return 2
    feat = QgsFeature()
    geom = QgsGeometry()
    n = 0
    while self.provider.nextFeature(feat):
      geom = feat.geometry()
      outFeat = QgsFeature()
      outFeat.setGeometry(geom)
      attrMap = {}
      for i in range(len(self.fields)):
        try:
          attrMap[i] = QVariant(self.data[i][n])
        except:
          attrMap[i] = QVariant('')
      outFeat.setAttributeMap(attrMap)
      writer.addFeature(outFeat)
      self.progressBar.setValue(n)
      n += 1
    del writer
    self.progressBar.reset()
    return 0



  def doDone(self): # Called when 'Done' button pressed
    question = 'The table contains unsaved changes. Do you really want to quit?'
    if not (self.isUnsaved and QMessageBox.question(self, 'Table Manager', question, QMessageBox.Yes, QMessageBox.No) == QMessageBox.No):
      self.close()



  def restoreCfg(self): # Restores previous session configuration or, if fails, init defaul values
    settings = QSettings()
    self.restoreGeometry(settings.value('/Plugin-TableManager/geometry').toByteArray())
    self.lastDirectory = settings.value('/Plugin-TableManager/lastDirectory', QVariant('.')).toString()
    self.lastEncoding = settings.value('/Plugin-TableManager/lastEncoding', QVariant('UTF-8')).toString()



  def saveCfg(self): # Saves configuration
    settings = QSettings()
    settings.setValue('/Plugin-TableManager/geometry', QVariant(self.saveGeometry()))
    settings.setValue('/Plugin-TableManager/lastDirectory', QVariant(self.lastDirectory))
    settings.setValue('/Plugin-TableManager/lastEncoding', QVariant(self.lastEncoding))
